/*
This work is part of the Core Imaging Library developed by
Visual Analytics and Imaging System Group of the Science Technology
Facilities Council, STFC

Copyright 2003 Hamish Carr
Copyright 2017 Srikanth Nagella, Edoardo Pasca

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
#ifndef WIN_UTILS_H
#define WIN_UTILS_H
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>


#ifdef __cplusplus
extern "C" {
#endif
	int gettimeofday(struct timeval * tp, struct timezone * tzp);
#ifdef __cplusplus
}
#endif

#endif

