echo $CIL_VERSION
#if [ -z "$CIL_VERSION" ]; then
#    echo "Need to set CIL_VERSION"
#    exit 1
#fi  
#export CIL_VERSION=0.9.1



mkdir ${SRC_DIR}/build
cp -rv ${RECIPE_DIR}/../../Core/ ${SRC_DIR}/build
mkdir ${SRC_DIR}/build/build
cd ${SRC_DIR}/build/build
cmake -G "Unix Makefiles" -DLIBRARY_LIB="${CONDA_PREFIX}/lib" -DLIBRARY_INC="${CONDA_PREFIX}" -DCMAKE_INSTALL_PREFIX="${PREFIX}" -DINSTALL_LIB_DIR="${PREFIX}/lib" -DINSTALL_BIN_DIR="${PREFIX}/bin" -DINSTALL_INCLUDE_DIR="${PREFIX}/include" --debug-output ../Core

#cmake -G "Unix Makefiles" -DLIBRARY_LIB="%CONDA_PREFIX%\lib" -DLIBRARY_INC="%CONDA_PREFIX%" -DCMAKE_INSTALL_PREFIX="%PREFIX%\Library" -DINSTALL_LIB_DIR="%PREFIX%\Library\lib" -DINSTALL_BIN_DIR="%PREFIX%\Library\bin" -DINSTALL_INCLUDE_DIR="%PREFIX%\Library\include" "%SRC_DIR%\build" 

make -j1 VERBOSE=1
make install
